/**
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.desktop.sample;

import cz.xelfi.quoridor.webidor.Game;
import cz.xelfi.quoridor.webidor.GameStatus;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
//import org.openide.util.ImageUtilities;
import org.netbeans.api.settings.ConvertAsProperties;

/**
 * Top component which displays the board.
 */
@ConvertAsProperties(dtd = "-//cz.xelfi.quoridor.desktop.sample//Kuk//EN",
autostore = false)
public final class KukTopComponent extends TopComponent {

    /** path to the icon used by the component and its open action */
//    static final String ICON_PATH = "SET/PATH/TO/ICON/HERE";
    private static final String PREFERRED_ID = "KukTopComponent";

    private String id;
    private String player;

    public KukTopComponent() {
        initComponents();
    }
    
    public void setGameId(String id) {
        this.id = id;
        final Game g = Quoridor.getDefault().getGame(id);
        boardPane1.setBoard(g.getBoard());
        player = currentPlayer(g);
        final boolean edit = Quoridor.getDefault().user().equals(player);
        boardPane1.setEditable(edit);
        setName(id);
        setHtmlDisplayName(
            edit ? NbBundle.getMessage(KukTopComponent.class, "CTL_GamePlay",
                g.getId().getWhite(), g.getId().getBlack()
            ) :
            NbBundle.getMessage(KukTopComponent.class, "CTL_Game",
                g.getId().getWhite(), g.getId().getBlack()
            )
        );
        setToolTipText(
            NbBundle.getMessage(KukTopComponent.class, "HINT_KukTopComponent",
                g.getId().getWhite(), g.getId().getBlack()
            )
        );
    }
    
    private static String currentPlayer(Game g) {
        if (g.getId().getStatus() == GameStatus.whiteMove) {
            return g.getId().getWhite();
        }
        if (g.getId().getStatus() == GameStatus.blackMove) {
            return g.getId().getBlack();
        }
        return null;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        boardPane1 = new cz.xelfi.quoridor.visidor.BoardPane();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();

        org.openide.awt.Mnemonics.setLocalizedText(jLabel1, org.openide.util.NbBundle.getMessage(KukTopComponent.class, "KukTopComponent.jLabel1.text")); // NOI18N

        boardPane1.addBoardListener(new cz.xelfi.quoridor.visidor.BoardListener() {
            public void boardChanged(cz.xelfi.quoridor.visidor.BoardEvent evt) {
                boardPane1BoardChanged(evt);
            }
        });

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cz/xelfi/quoridor/desktop/sample/black.png"))); // NOI18N
        org.openide.awt.Mnemonics.setLocalizedText(jButton1, org.openide.util.NbBundle.getMessage(KukTopComponent.class, "KukTopComponent.jButton1.text")); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cz/xelfi/quoridor/desktop/sample/white.png"))); // NOI18N
        org.openide.awt.Mnemonics.setLocalizedText(jButton2, org.openide.util.NbBundle.getMessage(KukTopComponent.class, "KukTopComponent.jButton2.text")); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jButton1)
                .addContainerGap(225, Short.MAX_VALUE))
            .addGroup(layout.createSequentialGroup()
                .addComponent(jButton2)
                .addContainerGap())
            .addComponent(boardPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 312, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jButton1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(boardPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 282, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButton2))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        setGameId(id);
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        setGameId(id);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void boardPane1BoardChanged(cz.xelfi.quoridor.visidor.BoardEvent evt) {//GEN-FIRST:event_boardPane1BoardChanged
        if (player != null) {
            Quoridor.getDefault().move(id, player, evt.getMove());
            setGameId(id);
        } else {
            boardPane1.setEditable(false);
        }
    }//GEN-LAST:event_boardPane1BoardChanged

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private cz.xelfi.quoridor.visidor.BoardPane boardPane1;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    // End of variables declaration//GEN-END:variables

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }

    @Override
    public void componentOpened() {
        // TODO add custom code on component opening
    }

    @Override
    public void componentClosed() {
        // TODO add custom code on component closing
    }

    void writeProperties(java.util.Properties p) {
        // better to version settings since initial version as advocated at
        // http://wiki.apidesign.org/wiki/PropertyFiles
        p.setProperty("version", "1.0");
        if (id != null) {
            p.setProperty("id", id);
        }
    }

    final void readProperties(java.util.Properties p) {
        String version = p.getProperty("version");
        String myId = p.getProperty("id");
        if (myId != null) {
            setGameId(myId);
        }
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }
}
