/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor;

import cz.xelfi.quoridor.Fence.Orientation;
import cz.xelfi.quoridor.Player.Direction;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class MoveTest {

    public MoveTest() {
    }

    @Test
    public void testNorth() throws Exception {
        checkValueOf("N", Move.NORTH);
    }
    @Test
    public void testSouth() throws Exception {
        checkValueOf("S", Move.SOUTH);
    }
    @Test
    public void testEast() throws Exception {
        checkValueOf("E", Move.EAST);
    }
    @Test
    public void testWest() throws Exception {
        checkValueOf("W", Move.WEST);
    }
    @Test
    public void testRegisn() throws Exception {
        checkValueOf("RESIGN", Move.RESIGN);
    }
    @Test
    public void testJumps() throws Exception {
        for (Direction d1 : Direction.values()) {
            for (Direction d2 : Direction.values()) {
                checkValueOf("" + d1.name().charAt(0) + d2.name().charAt(0), Move.jump(d1, d2));
            }
        }
    }
    @Test
    public void testFences() throws Exception {
        for (int i = 1; i <= 8; i++) {
            for (int j = 0; i < 8; i++) {
                String h = "H" + (char)('A'+ j) + i;
                String v = "V" + (char)('A'+ j) + i;

                Move hm = Move.fence((char) ('A' + j), i, Orientation.HORIZONTAL);
                Move vm = Move.fence((char) ('A' + j), i, Orientation.VERTICAL);

                checkValueOf(h, hm);
                checkValueOf(v, vm);
            }
        }
    }

    private static void checkValueOf(String txt, Move move) throws Exception {
        assertEquals("toString", txt, move.toString());
        assertEquals("valueOf", move, Move.valueOf(txt));
    }

}